using System;
using System.Collections.Generic;
using System.Text;


namespace RakabrotPluginInterface
{

    namespace RNumericImage
    {
        public class LineClipper
        {
            int Width;
            int Height;
            //Functions for clipping a 2D line to the screen, which is the rectangle (0,0)-(w,h)
            //This is the Cohen-Sutherland Clipping Algorithm
            //Each of 9 regions gets an outcode, based on if it's at the top, bottom, left or right of the screen
            // 1001 1000 1010  9 8 10
            // 0001 0000 0010  1 0 2
            // 0101 0100 0110  5 4 6
            //int findregion returns which of the 9 regions a point is in, void clipline does the actual clipping
            public LineClipper(int nWidth, int nHeight)
            {
                Width = nWidth;
                Height = nHeight;
            }
            public int findRegion(int x, int y)
            {
                int code = 0;
                if (y >= Height)
                    code |= 1; //top
                else if (y < 0)
                    code |= 2; //bottom
                if (x >= Width)
                    code |= 4; //right
                else if (x < 0)
                    code |= 8; //left
                return (code);
            }
            public bool clipLine(int x1, int y1, int x2, int y2,
                out  int x3, out  int y3, out  int x4, out int y4)
            {
                int code1, code2, codeout;
                bool accept = false, done = false;
                code1 = findRegion(x1, y1); //the region outcodes for the endpoints
                code2 = findRegion(x2, y2);
                do  //In theory, this can never end up in an infinite loop, it'll always come in one of the trivial cases eventually
                {
                    if (((code1 | code2) == 0)) accept = done = true;  //accept because both endpoints are in screen or on the border, trivial accept
                    else if ((code1 & code2) != 0) done = true; //the line isn't visible on screen, trivial reject
                    else  //if no trivial reject or accept, continue the loop
                    {
                        int x, y;
                        codeout = (code1 != 0) ? code1 : code2;
                        if ((codeout & 1) != 0) //top
                        {
                            x = x1 + (x2 - x1) * (Height - y1) / (y2 - y1);
                            y = Height - 1;
                        }
                        else if ((codeout & 2) != 0) //bottom
                        {
                            x = x1 + (x2 - x1) * -y1 / (y2 - y1);
                            y = 0;
                        }
                        else if ((codeout & 4) != 0) //right
                        {
                            y = y1 + (y2 - y1) * (Width - x1) / (x2 - x1);
                            x = Width - 1;
                        }
                        else //left
                        {
                            y = y1 + (y2 - y1) * -x1 / (x2 - x1);
                            x = 0;
                        }
                        if (codeout == code1) //first endpoint was clipped
                        {
                            x1 = x; y1 = y;
                            code1 = findRegion(x1, y1);
                        }
                        else //second endpoint was clipped
                        {
                            x2 = x; y2 = y;
                            code2 = findRegion(x2, y2);
                        }
                    }
                }
                while (!done);

                if (accept)
                {
                    x3 = x1;
                    x4 = x2;
                    y3 = y1;
                    y4 = y2;
                    return true;
                }
                else
                {
                    x3 = x4 = y3 = y4 = 0;
                    return false;
                }
            }

        }
    }
}