using System;
using System.IO;

using System.Collections.Generic;

using System.Text;
using System.ComponentModel;
using RakabrotPluginInterface.RMath;

namespace RakabrotPluginInterface

{
	/// <summary>
	/// 
	/// </summary>
	public class RCalculationInfo
	{
        public enum RFractalType { Mandelbrot, Julia }



        bool mBuddhabrotInside;
        bool mBuddhabrotOutside;
        bool mCalculationPathInside;
        bool mCalculationPathOutside;

        bool mTimeSpentInside;
        bool mTimeSpentOutside;

        bool mMSet;

        RFractalType mType;
        double   mJuliaSeedReal;
        double mJuliaSeedImag;
        Complex mJuliaSeed;
        double mTimeSpentConstant;
        

		public RCalculationInfo()
		{
			// 
			// TODO: Add constructor logic here
			//
            mBuddhabrotInside = false;
            mBuddhabrotOutside = true;
            mCalculationPathInside = false;
            mCalculationPathOutside = true;
            mTimeSpentInside = false;
            mTimeSpentOutside = true;
            mMSet = true;
            JuliaSeedImag = 0;
            JuliaSeedReal = 1;
            mTimeSpentConstant = 1000;

        }
        [BrowsableAttribute(true), CategoryAttribute("Time Spent")]
        public double TimeSpentConstant
        {
            get
            {
                return mTimeSpentConstant;
            }
            set
            {
                mTimeSpentConstant = value;
                
            }
        }


        [BrowsableAttribute(true), CategoryAttribute("Initial fractal")]
        public double JuliaSeedReal
        {
            get
            {
                return mJuliaSeedReal;
            }
            set
            {
                mJuliaSeedReal = value;
                mJuliaSeed.SetReal(JuliaSeedReal);
            }
        }
        [BrowsableAttribute(true), CategoryAttribute("Initial fractal")]
        public double JuliaSeedImag
        {
            get
            {
                return mJuliaSeedImag;
            }
            set
            {
                mJuliaSeedImag = value;
                mJuliaSeed.SetImag(JuliaSeedImag);
            }
        }

        public Complex GetJuliaSeed() { return mJuliaSeed;}

        [BrowsableAttribute(true), CategoryAttribute("Buddhabrot")]
        public bool BuddhabrotOutside
        {
            get
            {
                return mBuddhabrotOutside;
            }
            set
            {
                mBuddhabrotOutside = value;
            }
        }

        [BrowsableAttribute(true), CategoryAttribute("Initial fractal")]
        public RFractalType FractalType
        {
            get
            {
                return mType;
            }
            set
            {
                mType = value;
            }
        }
        [BrowsableAttribute(true), CategoryAttribute("Calculation Path")]
        public bool CalculationPathOutside
        {
            get
            {
                return mCalculationPathOutside;
            }
            set
            {
                mCalculationPathOutside = value;
            }
        }

        [BrowsableAttribute(true), CategoryAttribute("Time Spent")]
        public bool TimeSpentOutside
        {
            get
            {
                return mTimeSpentOutside;
            }
            set
            {
                mTimeSpentOutside = value;
            }
        }


		public void SaveToFile(StreamWriter sw)
		{
			sw.WriteLine("Calculate Inside");
			sw.WriteLine(mBuddhabrotInside);
			sw.WriteLine("Calculate OutSide");
			sw.WriteLine(mBuddhabrotOutside);
			sw.WriteLine("Calculate M-Set");
			sw.WriteLine(mMSet);

		}

		/// <summary>
		/// Calculate and save Inside type Buddhabrot fractal
		/// </summary>
        [BrowsableAttribute(true), CategoryAttribute("Buddhabrot")]
        public bool BuddhabrotInside
        {
            get
            {
                return mBuddhabrotInside;
            }
            set
            {
                mBuddhabrotInside = value;
            }
        }
        [BrowsableAttribute(true), CategoryAttribute("Calculation Path")]
        public bool CalculationPathInside
        {
            get
            {
                return mCalculationPathInside;
            }
            set
            {
                mCalculationPathInside = value;
            }

        }
        [BrowsableAttribute(true), CategoryAttribute("Time Spent")]
        public bool TimeSpentInside
        {
            get
            {
                return mTimeSpentInside;
            }
            set
            {
                mTimeSpentInside = value;
            }

        }

		public bool MSet
		{
			get
			{
				return mMSet;
			}
			set
			{
				mMSet = value;
			}
		}
	}
}
