using System;
using System.IO;
using RakabrotPluginInterface.RCommon;
using RakabrotPluginInterface.RMath;

using RakabrotPluginInterface.RNumericImage;



namespace RakabrotPluginInterface
{
    /// <summary>
	/// 
	/// </summary>

    public abstract class RFractalFormulaAbstract
	{
        public enum RInitFunction { ident, sin,sinh, cos,cosh, sqr, log, sqrt, tang,tanh, exp,z };

        protected int               mBailout;
		protected int				mBailoutSqr;
        protected int               mMaxIterations;


        protected RFractalFormulaAbstract.RInitFunction mInitFunction;


		protected string			mName;
		protected string			mFormulaString;

        protected RGridArea         mMandelbortArea;
		protected RNumericImageAbstract		mMandelbrot;

		protected RGridArea         mBuddhaArea;

        protected RNumericImageAbstract mBuddhabrotOut;
        protected RNumericImageAbstract mBuddhabrotIn;

        protected RNumericImageAbstract mCalculationPathOut;
        protected RNumericImageAbstract mCalculationPathIn;

        protected RNumericImageAbstract mTimeSpentOut;
        protected RNumericImageAbstract mTimeSpentIn;


		protected Complex		    mPixel;
		protected uint				mCounter;
		protected RCalculationInfo  mCalculationInfo;
		protected RPathArray	    mCalcPath;
        protected RAreaOptimizationArray mOptimization;
        protected RProgress         mCalcProgress;
        protected string            mLogFile;


        public  Complex Pixel()
        {
            if (mCalculationInfo.FractalType == RCalculationInfo.RFractalType.Mandelbrot)
            {
                return mPixel;
            }
            else 
            {
                return mCalculationInfo.GetJuliaSeed();

            }
        }
		public RFractalFormulaAbstract()
		{
			// 
			// TODO: Add constructor logic here
			//
			Bailout				= 128;
			mMaxIterations		= 256;
			mCalculationInfo	=  new RCalculationInfo();
			mCalcPath           =  new RPathArray(mMaxIterations);
            
//            Directory.CreateDirectory( + "\\Log");
//            mLogFile = RakaBrot.MainWindow.StartDirectory + "\\Log\\FraktaliLog.txt";
		}

        public RFractalFormulaAbstract(string StartDirectory)
        {
            // 
            // TODO: Add constructor logic here
            //
            Bailout = 128;
            mMaxIterations = 256;
            mCalculationInfo = new RCalculationInfo();
            mCalcPath = new RPathArray(mMaxIterations);
            Directory.CreateDirectory(StartDirectory + "\\Log");
            mLogFile = StartDirectory + "\\Log\\FraktaliLog.txt";
        }



        public void SetProgress(RProgress nProgress)
        {
            mCalcProgress = nProgress;
        }
        protected Complex InitZ(Complex Pixel)
        {
            return GetFuncValue(Pixel, mInitFunction);
        }
        protected Complex GetFuncValue(Complex Pixel, RInitFunction FuncType)
        {
            switch (FuncType)
            {
                case RInitFunction.cos: return Complex.Cos(Pixel);
                case RInitFunction.cosh: return Complex.Cosh(Pixel);
                case RInitFunction.sin: return Complex.Sin(Pixel);
                case RInitFunction.sinh: return Complex.Sinh(Pixel);
                case RInitFunction.log: return Complex.Log(Pixel);
                case RInitFunction.sqr: return Pixel * Pixel;
                case RInitFunction.sqrt: return Complex.Pow(Pixel, new Complex(0.5, 0));
                case RInitFunction.ident: return new Complex(1, 0);
                case RInitFunction.tang: return Complex.Tan(Pixel);
                case RInitFunction.tanh: return Complex.Tanh(Pixel);
                case RInitFunction.exp: return Complex.Exp(Pixel);
                case RInitFunction.z: return Pixel;

            }
            return new Complex(1, 0);
        }
        public int Bailout
        {
            get
            {
                return mBailout;
            }
            set
            {
                mBailout = value;
                mBailoutSqr = mBailout * mBailout;
            }
        }
        public RInitFunction InitFunction
        {
            get
            {
                return mInitFunction;
            }
            set
            {
                mInitFunction = value;
            }
        }

		public int MaxIterations
		{
			get
			{
				return mMaxIterations;
			}
			set
			{
				mMaxIterations		= value;
				mCalcPath           =  new RPathArray(mMaxIterations);
			}
		}

		public virtual void SaveToFile(StreamWriter sw, string ExtraInfo)
		{
	
			sw.WriteLine("BailOut");
			sw.WriteLine(mBailout);

			sw.WriteLine("MaxIterations");
			sw.WriteLine(mBailout);
				
		}

		public virtual void LoadFromFile(StreamReader sr)
		{
			String Line; 
			

			Line     = sr.ReadLine();
			mBailout = Convert.ToInt32(Line );

			Line		    = sr.ReadLine();
			mMaxIterations  = Convert.ToInt32(Line );
		
		}
		public virtual string GetName()
		{
			return mName;
		}
		public virtual string GetFormulaString()
		{
			return mFormulaString;
		}

        public virtual void SetmMandelBrot(RNumericImageAbstract nMandelBrot)
        {
            mMandelbrot = nMandelBrot;
        }
        public virtual void SetOptimization(RAreaOptimizationArray nOptimization)
        {
            mOptimization = nOptimization;
        }
        public virtual void SetmBuddhabrotOut(RNumericImageAbstract nBuddhabrotOut)
        {
            mBuddhabrotOut = nBuddhabrotOut;
        }
        public virtual void SetmBuddhabrotIn(RNumericImageAbstract nBuddhabrotIn)
        {
            mBuddhabrotIn = nBuddhabrotIn;
        }
        public virtual void SetmCalculationPathOut(RNumericImageAbstract nRakabrotOut)
        {
            mCalculationPathOut = nRakabrotOut;
        }
        public virtual void SetmCalculationPathIn(RNumericImageAbstract nRakabrotIn)
        {
            mCalculationPathIn = nRakabrotIn;
        }

        public virtual void SetmTimeSpentOut(RNumericImageAbstract nRakabrotOut)
        {
            mTimeSpentOut = nRakabrotOut;
        }
        public virtual void SetmTimeSpentIn(RNumericImageAbstract nRakabrotIn)
        {
            mTimeSpentIn = nRakabrotIn;
        }

        public void Calculate()
		{
                
			double	StepReal = mMandelbortArea.StepX();
			double	StepImag = mMandelbortArea.StepY();
            StreamWriter sw = new StreamWriter(mLogFile);
            sw.AutoFlush = true; 
			Complex Z = new Complex();
            int     Index;


            if (mCalcProgress != null)
            {
                mCalcProgress.progressBar1.Minimum = 0;
                mCalcProgress.progressBar1.Maximum = mMandelbrot.Height;
                mCalcProgress.progressBar1.Step = 1;
                mCalcProgress.progressBar1.Value = 0;
            }
            mPixel.SetImag(mMandelbortArea.Top);
            for (int j = 0; j < mMandelbortArea.Height; j++)
			{
                if (mCalcProgress != null)
                {
                    mCalcProgress.progressBar1.PerformStep();
                    mCalcProgress.ProgressTekst.Text = j + "/" + mMandelbrot.Height;
                    mCalcProgress.progressBar1.Refresh();
                    mCalcProgress.ProgressTekst.Refresh();


                }
                mPixel.SetReal(mMandelbortArea.Left);
                for (int i = 0; i < mMandelbortArea.Width; i++)
			{
              if ((Index = mOptimization.Inside(mPixel.Real, mPixel.Imag)) != -1)
                    {
                        double Diff =  mOptimization[Index].Right - mPixel.Real;
                        int steps = (int)((Diff  )/ StepReal);
                        if (steps <= 0) steps = 1;
                        i +=   steps-1;
                        mPixel.AddReal(StepReal*steps);


                    }else{
				    	Z = mPixel;
					    Init(Z);
					    Loop(Z);
                        ChangeImages(i, j);
			    	    mPixel.AddReal(StepReal);
                    }
                    
				}
             mPixel.AddImag(StepImag);
            if(j%20 == 0) sw.WriteLine(j);
         }
         sw.Close();
         if (mCalculationInfo.CalculationPathInside)
             mCalculationPathIn.NumericImageScale(800);
         if (mCalculationInfo.CalculationPathOutside)
             mCalculationPathOut.NumericImageScale(800);
                    
     }
		protected abstract  void Init(Complex Z);
		protected abstract  uint  Loop(Complex Z);
		protected virtual   void ChangeImages(int x, int y)
		{
			if(mCalculationInfo.MSet)
					mMandelbrot[x,y] = mCounter;
                if (mCalculationInfo.BuddhabrotInside)
                {
                    if (mCounter == mMaxIterations)
                        mBuddhabrotIn.AddPathArray(mCalcPath, mBuddhaArea.GetArea());
                }
                if (mCalculationInfo.BuddhabrotOutside)
                {
                    if (mCounter < mMaxIterations)
                        mBuddhabrotOut.AddPathArray(mCalcPath, mBuddhaArea.GetArea());
                }
                if (mCalculationInfo.CalculationPathInside)
                {
                    if (mCounter == mMaxIterations)
                        mCalculationPathIn.AddPathArrayLine(mCalcPath, mBuddhaArea.GetArea(), 1 / 320, false);
                    
                }
                if (mCalculationInfo.CalculationPathOutside)
                {
                    if (mCounter < mMaxIterations)
                        mCalculationPathOut.AddPathArrayLine(mCalcPath, mBuddhaArea.GetArea(), 1 / 320, false);
                        
                }
                if (mCalculationInfo.TimeSpentInside)
                {
                    if (mCounter == mMaxIterations)
                        mTimeSpentIn.AddPathArrayLine(mCalcPath, mBuddhaArea.GetArea(), mCalculationInfo.TimeSpentConstant, true);

                }
                if (mCalculationInfo.TimeSpentOutside)
                {
                    if (mCounter < mMaxIterations)
                        mTimeSpentOut.AddPathArrayLine(mCalcPath, mBuddhaArea.GetArea(), mCalculationInfo.TimeSpentConstant, true);

                }


            }
		protected virtual   bool CheckBailout(Complex Z)
		{	
				Complex Temp = Z*Z.Conjugate;
				if(Temp.Real > mBailoutSqr)
					return true;
				else
					return false;
		}

		public void SetGridArea(RGridArea nGridArea)
		{
			mMandelbortArea = nGridArea;
		}
		public void SetBuddhaGridArea(RGridArea nGridArea)
		{
			mBuddhaArea = nGridArea;
		}

		public void SetCalculationInfo(RCalculationInfo nCalculationInfo)
		{
			mCalculationInfo = nCalculationInfo;
		}
	}

}
