using System;
using System.IO;
using System.Drawing;
using RakabrotPluginInterface.RCommon;
using System;

using Inferno.Utilities;
namespace RakabrotPluginInterface
{

    namespace RNumericImage
    {
        /// <summary>
        /// 
        /// </summary>
        /// 
        public abstract class RNumericImageAbstract
        {
            protected RGrid mGrid;
            protected double[] mData;
            protected Bitmap mBitmap;
            protected bool freshBitmap;
            protected string mBitmapFileName;
            protected string mDataFileName;
            public LineClipper mLineClipper;

            protected MarkerList mGradient;


            double mMaxValue;
            bool freshMaxValue;

            public MarkerList GetGradient() { return mGradient; }

            public MarkerList Gradient
            {
                get
                {
                    return mGradient;
                }
                set
                {
                    mGradient = value;
                }
            }
            public double MaxValue
            {
                get
                {
                    if (!freshMaxValue)
                    {
                        CalculateMaxValue();
                    }
                    return mMaxValue;
                }
            }
            private void CalculateMaxValue()
            {
                mMaxValue = double.MinValue;
                int Length;
                Length = mGrid.Width * mGrid.Height;
                for (int i = 0; i < Length; i++)
                    if (mMaxValue < mData[i])
                        mMaxValue = mData[i];


            }


            public RNumericImageAbstract()
            {
                freshBitmap = false;
                mGradient = new MarkerList();


            }
            public virtual void InitImageSize(int nWidth, int nHeight)
            {
                mGrid = new RGrid();
                mGrid.Width = nWidth;
                mGrid.Height = nHeight;
                mData = new double[mGrid.Width * mGrid.Height];
                freshBitmap = false;
                mLineClipper = new LineClipper(mGrid.Width, mGrid.Height);


            }
            public RNumericImageAbstract(int nWidth, int nHeight)
            {
                // 
                // TODO: Add constructor logic here
                //
                InitImageSize(nWidth, nHeight);
                mBitmapFileName = "c:\\fraktali\\test.bmp";
                mDataFileName = "C:\\fraktali\\NumericImage.dat";
                mGradient = new MarkerList();
            }
            public RNumericImageAbstract(RGrid nGrid)
            {
                InitImageSize(nGrid.Width, nGrid.Height);
                mBitmapFileName = "C:\\fraktali\\test.bmp";
                mDataFileName = "C:\\fraktali\\NumericImage.dat";
                mGradient = new MarkerList();
            }
            public RNumericImageAbstract(RGrid nGrid, MarkerList nGradient)
            {
                InitImageSize(nGrid.Width, nGrid.Height);

                mBitmapFileName = "C:\\fraktali\\test.bmp";
                mDataFileName = "c:\\fraktali\\NumericImage.dat";
                mGradient = nGradient;

            }
            
            public abstract bool GetChange();

            protected bool IsBitmapFresh()
            {
                bool result = !(GetChange());

                result = result && freshBitmap;
                freshBitmap = true;
                return result;
            }
            public virtual void DrawImage(Graphics g)
            {

                if (!IsBitmapFresh())
                {
                    CalcBitmap();
                }
                g.DrawImage(
                    mBitmap,
                    0,
                    0,
                    mBitmap.Width,
                    mBitmap.Height);

            }
            public abstract void LoadDataFromFile(string FileName);
            public void LoadDataFromFilePart(string FileName)
            {

                FileStream fs;

                fs = new FileStream(FileName, FileMode.Open, FileAccess.Read);
                BinaryReader r = new BinaryReader(fs);

                mGrid.Width = r.ReadInt16() / 4;
                mGrid.Height = r.ReadInt16() / 4;

                mData = new double[mGrid.Width * mGrid.Height];
                mLineClipper = new LineClipper(mGrid.Width, mGrid.Height);


                for (int i = 0; i < mGrid.Width * 4; i++)
                    for (int j = 0; j < mGrid.Height * 4; j++)
                    {
                        this[i / 4, j / 4] = r.ReadDouble();
                    }

                freshBitmap = false;


            }
            public void SaveDataToFile()
            {
                SaveDataToFile(mDataFileName);
            }

            public void SaveDataToFile(string FileName)
            {
                FileStream fs;
                fs = new FileStream(FileName, FileMode.Create, FileAccess.Write);

                BinaryWriter w = new BinaryWriter(fs);

                w.Write((short)mGrid.Width);
                w.Write((short)mGrid.Height);

                for (int i = 0; i < mGrid.Width; i++)
                    for (int j = 0; j < mGrid.Height; j++)
                    {
                        w.Write((short)this[i, j]);
                    }


            }
            public double this[int i, int j]
            {
                get
                {
                    return mData[i + j * mGrid.Width];
                }
                set
                {
                    mData[i + j * mGrid.Width] = value;
                    freshBitmap = false;

                }
            }
            public double this[int i]
            {
                get
                {
                    return mData[i];
                }
                set
                {
                    mData[i] = value;
                    freshBitmap = false;

                }
            }

            public void SetGrid(RGrid nGrid)
            {
                mGrid = nGrid;
                mData = new double[mGrid.Width * mGrid.Width];
                mLineClipper = new LineClipper(mGrid.Width, mGrid.Height);

                for (int i = 0; i < mGrid.Width; i++)
                    for (int j = 0; j < mGrid.Height; j++)
                    {
                        this[i, j] = (double)0; ;// (i * 2 + j * 3);
                    }

            }
            public abstract void CalcBitmap();


            public int Width
            {
                get
                {
                    return mGrid.Width;
                }
                set
                {
                    mGrid.Width = value;
                }
            }

            public int Height
            {
                get
                {
                    return mGrid.Height;
                }
                set
                {
                    mGrid.Height = value;

                }
            }
            public void SetToInitValue(uint InitValue)
            {
                int Size = mGrid.Height * mGrid.Width;
                for (int i = 0; i < Size; i++)
                    mData[i] = InitValue;
            }
            public void AddPathArray(RPathArray nPath, RArea nArea)
            {

                // Ovde Postoji Optimizacija
                nPath.ResetCurrent();
                double Real, Imag;

                double StepX = mGrid.Width / nArea.Width;
                double StepY = mGrid.Height / nArea.Height;

                int indexX;
                int indexY;
                while (nPath.GetCurrent(out Real, out Imag))
                {
                    indexX = (int)((Real - nArea.Left) * StepX);
                    if ((indexX >= 0) && (indexX < mGrid.Width))
                    {
                        indexY = (int)((Imag - nArea.Top) * StepY);
                        if ((indexY >= 0) && (indexY < mGrid.Height))
                        {
                            this[indexX, indexY] = this[indexX, indexY] + 1;
                            //                        ClipLine(indexX, indexY, indexX, indexY, 1);
                        }

                    }

                }
                freshBitmap = false;

            }
            public void AddPathArrayLine(RPathArray nPath, RArea nArea, double Koef, bool useDistance)
            {

                // Ovde Postoji Optimizacija
                nPath.ResetCurrent();
                double Real, Imag;
                double tReal, tImag;

                double StepX = mGrid.Width / nArea.Width;
                double StepY = mGrid.Height / nArea.Height;
                double Length;
                int indexX;
                int indexY;
                double ValuePX = 0;
                double ValuePY = 0;
                int indexPX = 0;
                int indexPY = 0;
                int counter = 0;
                double Length1;
                double Length2;


                while (nPath.GetCurrent(out Real, out Imag))
                {
                    indexX = (int)((Real - nArea.Left) * StepX);
                    indexY = (int)((Imag - nArea.Top) * StepY);




                    if (counter == 0)
                    {

                        indexPX = indexX;
                        indexPY = indexY;
                        ValuePX = Real;
                        ValuePY = Imag;

                    }


                    /*                    Length1 = Math.Sqrt((indexPX - indexX) * (indexPX - indexX)
                                                                             + (indexPY - indexY) * (indexPY - indexY));
                    */
                    Length2 = (Real - ValuePX) * (Real - ValuePX)
                             + (Imag - ValuePY) * (Imag - ValuePY);

                    Length = Length2 * 1000;
                    //                    Length = 1000000 * Length;
                    if (Length < 0.1)
                        Length = 1;
                    else
                        Length = Koef / Length;

                    if (useDistance)
                        ClipLine(indexPX, indexPY, indexX, indexY, Length);
                    else
                        ClipLine(indexPX, indexPY, indexX, indexY, 1);

                    indexPX = indexX;
                    indexPY = indexY;
                    ValuePX = Real;
                    ValuePY = Imag;

                    counter++;
                }
                freshBitmap = false;

            }



            void ClipLine(int x1, int y1, int x2, int y2, double inc)
            {
                int ClipX1 = 0, ClipY1 = 0;
                int ClipX2 = 0, ClipY2 = 0;
                mLineClipper.clipLine(x1, y1, x2, y2,
                                      out ClipX1, out ClipY1,
                                      out ClipX2, out ClipY2);

                DrawLine(ClipX1, ClipY1, ClipX2, ClipY2, inc);
            }
            public void DrawLine(int x1, int y1, int x2, int y2, double inc)
            {
                int deltax, deltay;
                int x, y;
                int xinc1, xinc2;
                int yinc1, yinc2;
                int den, num, numadd, numpixels;
                int index, indexY;
                int incY1;
                int incY2;

                deltax = Math.Abs(x2 - x1);        // razlika  x eva
                deltay = Math.Abs(y2 - y1);        // razlika  x eva

                x = x1;                       // postavljanje pocetnog pixela za X
                y = y1;                       // postavljanje pocetnog pixela za Y

                if (x2 >= x1)                 // X - evi rastu
                {
                    xinc1 = 1;
                    xinc2 = 1;
                }
                else                          // X - evi opadaju
                {
                    xinc1 = -1;
                    xinc2 = -1;
                }

                if (y2 >= y1)                 // Y - evi rastu
                {
                    yinc1 = 1;
                    yinc2 = 1;
                    incY1 = mGrid.Width;
                    incY2 = mGrid.Width;
                }
                else                          // X - evi opadaju
                {
                    yinc1 = -1;
                    yinc2 = -1;
                    incY1 = -mGrid.Width;
                    incY2 = -mGrid.Width;
                }

                if (deltax >= deltay)         // Postoji bar jedan vise X
                {
                    xinc1 = 0;                  // ne menjaj X ako numerator >= denominatora
                    yinc2 = 0;                  // nemanjaj y pri svakoj itereciji
                    incY2 = 0;

                    den = deltax;
                    num = deltax / 2;
                    numadd = deltay;
                    numpixels = deltax;         // ima vise vrednosti za x nego za y
                }
                else                          // Postoji bar jedan vise Y
                {
                    xinc2 = 0;                  // nemanjaj x pri svakoj itereciji
                    yinc1 = 0;                  // ne menjaj X ako numerator >= denominatora
                    incY1 = 0;
                    den = deltay;
                    num = deltay / 2;
                    numadd = deltax;
                    numpixels = deltay;         // ima vise y vrednosti nego x
                }
                int curpixel;
                indexY = y * mGrid.Width;
                for (curpixel = 0; curpixel <= numpixels; curpixel++)
                {
                    //                this[x, y] = this[x, y] + 1;
                    this[x + indexY] = this[x + indexY] + inc;
                    num += numadd;              //  uvecaj numerator za vrh razlomka
                    if (num >= den)             // proveri da li je brojilac veci od imenioca
                    {
                        num -= den;               // izracunaj novu vrednost brojioca
                        x += xinc1;               // promeni x na odgovarajuci nacin
                        y += yinc1;               // promeni y na odgovarajuci nacin
                        indexY += incY1;
                    }
                    x += xinc2;                 // promeni x na odgovarajuci nacin
                    y += yinc2;                 // promeni y na odgovarajuci nacin
                    indexY += incY2;
                }
            }
//            public virtual RGradient GetGradient() { return null; }
/*            public virtual void SetGradient(RGradient nGradient)
            {
            }
 */
            public void SaveBitMap(string nBitmapFileName)
            {
                mBitmapFileName = nBitmapFileName;
                SaveBitMap();
            }

            public void SaveBitMap()
            {
                if (!IsBitmapFresh())
                {
                    CalcBitmap();
                }
                if (mBitmapFileName != "")
                    mBitmap.Save(mBitmapFileName);
            }

            public string BitmapFilename
            {
                get
                {
                    return mBitmapFileName;
                }
                set
                {
                    mBitmapFileName = value;
                }
            }
            public void NumericImageScale(int Scale)
            {
                for (int i = 0; i < mGrid.Width; i++)
                    for (int j = 0; j < mGrid.Height; j++)
                    {
                        //  this[i, j] = (uint)(this[i, j] / Scale);
                        this[i, j] = (uint)(this[i, j]);
                    }
            }
            /*
              public void ConvertBmpToData(Bitmap inputBitmap, RColorHolder iColorHolder)
              {


                  Color[] ColorPalete = new Color[50000];
                  Color CurrentColor;
                  int MaxIndex = 0;
                  bool ind = false;

                  uint i;

                  //            InitImageSize(inputBitmap.Width, inputBitmap.Height);


                  System.Drawing.ColorConverter Type;

                  Type = new ColorConverter();


                  System.Drawing.Imaging.BitmapData bitmapData = inputBitmap.LockBits(new Rectangle(0, 0, inputBitmap.Width, inputBitmap.Height),
                                                  System.Drawing.Imaging.ImageLockMode.ReadOnly,
                                                  inputBitmap.PixelFormat);

                  uint index = 0;

                  IntPtr pixels = bitmapData.Scan0;
                  //            Color CurrentColor;
                  unsafe
                  {

                      byte* pBits = (byte*)pixels.ToPointer();

                      for (int y = 0; y < mGrid.Height; y++)
                          for (int x = 0; x < mGrid.Width; x++)
                          {

                              //                    CurrentColor = inputBitmap.GetPixel(x, y);
                              ind = true;
                              CurrentColor = System.Drawing.Color.FromArgb((byte)255, pBits[index], pBits[index + 1], pBits[index + 2]);

                              this[x, y] = iColorHolder.GetColorIndex(CurrentColor);

                              if (bitmapData.PixelFormat
                                  == System.Drawing.Imaging.PixelFormat.Format32bppArgb)
                                  index += 4;
                              else
                                  index += 3;


                          }
                  }

                  inputBitmap.UnlockBits(bitmapData);
              } 
             */ 
           
        }
           
    }
}