using System;
using System.IO;
using RakabrotPluginInterface.RCommon;
using Rakabrot;
using RakabrotPluginInterface.RNumericImage;
using RakabrotPluginInterface;

namespace CRFractal
{
	/// <summary>
	/// 
	/// </summary>
	public class RFractal
	{	
		RGridArea			 	mInputGridArea;
		RGridArea				mOutputGridArea;
		RAreaOptimizationArray	mOptimization;

		RFractalFormulaAbstract mFormula;
        RNumericImageAbstract mMandelBrot;

        RNumericImageAbstract mBuddhabrotIn;
        RNumericImageAbstract mBuddhabrotOut;
        RNumericImageAbstract mCalculationPathIn;
        RNumericImageAbstract mCalculationPathOut;

        RNumericImageAbstract mTimeSpentIn;
        RNumericImageAbstract mTimeSpentOut;

		public RCalculationInfo	mCalculationInfo;
        private RakabrotPluginPluginHolder mPluginHolder;

        public void FillCombox(System.Windows.Forms.ComboBox nComboBox)
        {
            nComboBox.Items.Clear();
            for (int i = 0; i < mPluginHolder.MaxIndex; i++) {
                nComboBox.Items.Add(mPluginHolder.GetName(i));
            }
                        
        }
        public RFractal()
		{
			// 
			// TODO: Add constructor logic here
			//
			mInputGridArea		= new RGridArea();
			mOutputGridArea		= new RGridArea();
			mCalculationInfo	= new RCalculationInfo();
//            mFormula = new RFractalFormulaZPower();
            //            mFormula = new RFractalFormulaPowerEx();
            mPluginHolder = new RakabrotPluginPluginHolder(50);
            mPluginHolder.LoadPluginsFromDirectory(MainWindow.StartDirectory);

            
            SetFormula(mPluginHolder.GetName(0));

            //			mMandelBrot			= new RNumericImageAbstract(320,200);

            int     tmHeight, tmWidth;


            tmWidth = 320;

            tmHeight = 200;
            mMandelBrot = new RNumericImagePallete(tmWidth, tmHeight);


            mBuddhabrotOut = new RNumericImagePallete(tmWidth, tmHeight);
            mBuddhabrotIn = new RNumericImagePallete(tmWidth, tmHeight);

            mCalculationPathOut = new RNumericImagePallete(tmWidth, tmHeight);
            mCalculationPathIn = new RNumericImagePallete(tmWidth, tmHeight);

           
            mTimeSpentOut = new RNumericImagePallete(tmWidth, tmHeight);
            mTimeSpentIn = new RNumericImagePallete(tmWidth, tmHeight);


            mMandelBrot.SetGrid(mInputGridArea.Grid);

            mBuddhabrotOut.SetGrid(mOutputGridArea.Grid);
            mBuddhabrotIn.SetGrid(mOutputGridArea.Grid);

            mCalculationPathOut.SetGrid(mOutputGridArea.Grid);
            mCalculationPathIn.SetGrid(mOutputGridArea.Grid);

            mTimeSpentOut.SetGrid(mOutputGridArea.Grid);
            mTimeSpentIn.SetGrid(mOutputGridArea.Grid);

			mFormula.SetmMandelBrot(mMandelBrot);
			
			mFormula.SetGridArea(mInputGridArea);

            mFormula.SetCalculationInfo(mCalculationInfo);

            mOptimization = new RAreaOptimizationArray(150);
            mFormula.SetOptimization(mOptimization);
        }

        public void SetInutArea(RArea nArea)
        {
            mInputGridArea.Area = nArea;
            mFormula.SetGridArea(mInputGridArea);
        }
        public void SetOutputArea(RArea nArea)
        {
            mOutputGridArea.Area = nArea;
        }

        public void SetFormula(string FormulaType)
        {

            mFormula = mPluginHolder.GetFormula(FormulaType);
            /*            if (FormulaType == "ZPower")
                        {
                            mFormula = new RFractalFormulaZPower(MainWindow.StartDirectory);
                        }

                        if (FormulaType == "ZPowerEx")
                        {
                            mFormula = new RFractalFormulaPowerEx(MainWindow.StartDirectory);
                        }
                        if (FormulaType == "Sum")
                        {
                            mFormula = new RFractalFormulaSum(MainWindow.StartDirectory);
                        }
             */
            mFormula.SetCalculationInfo(mCalculationInfo);
            mOptimization = new RAreaOptimizationArray(150);
            mFormula.SetOptimization(mOptimization);
  
        }
        public void SetProgress(RProgress nProgress)
        {
            mFormula.SetProgress(nProgress);
        }

		protected     void InitBuddbrotNumericImages()
		{
            if (mCalculationInfo.MSet)
            {
//                mMandelBrot = new RNumericImageAbstract(mInputGridArea.Grid, mMandelBrot.GetGradient());
                mMandelBrot = new RNumericImagePallete(mInputGridArea.Grid, mMandelBrot.GetGradient());
                mMandelBrot.SetToInitValue(23);
                mFormula.SetmMandelBrot(mMandelBrot);
 
            }
            mFormula.SetGridArea(mInputGridArea);

            if (mCalculationInfo.BuddhabrotInside)
            {
                mBuddhabrotIn = new RNumericImagePallete(mOutputGridArea.Grid, mBuddhabrotIn.GetGradient());
                mBuddhabrotIn.SetToInitValue(0);

                mFormula.SetBuddhaGridArea(mOutputGridArea);
                mFormula.SetmBuddhabrotIn(mBuddhabrotIn);

            }
            if (mCalculationInfo.BuddhabrotOutside)
            {
                mBuddhabrotOut = new RNumericImagePallete(mOutputGridArea.Grid, mBuddhabrotOut.GetGradient());
                mBuddhabrotOut.SetToInitValue(0);

                mFormula.SetBuddhaGridArea(mOutputGridArea);
                mFormula.SetmBuddhabrotOut(mBuddhabrotOut);
            }
            if (mCalculationInfo.CalculationPathInside)
            {
                mCalculationPathIn = new RNumericImagePallete(mOutputGridArea.Grid, mBuddhabrotOut.GetGradient());
                mCalculationPathIn.SetToInitValue(0);

                mFormula.SetBuddhaGridArea(mOutputGridArea);
                mFormula.SetmCalculationPathIn(mCalculationPathIn);
                mCalculationPathIn.Gradient.Function = Inferno.Utilities.MarkerList.RFunctions.sqrt;


            }
            if (mCalculationInfo.CalculationPathOutside)
            {
                mCalculationPathOut = new RNumericImagePallete(mOutputGridArea.Grid, mBuddhabrotOut.GetGradient());
                mCalculationPathOut.SetToInitValue(0);

                mFormula.SetBuddhaGridArea(mOutputGridArea);
                mFormula.SetmCalculationPathOut(mCalculationPathOut);
                mCalculationPathOut.Gradient.Function = Inferno.Utilities.MarkerList.RFunctions.pow;
            }

            if (mCalculationInfo.TimeSpentInside)
            {
                mTimeSpentIn = new RNumericImagePallete(mOutputGridArea.Grid, mBuddhabrotOut.GetGradient());
                mTimeSpentIn.SetToInitValue(0);

                mFormula.SetBuddhaGridArea(mOutputGridArea);
                mFormula.SetmTimeSpentIn(mTimeSpentIn);
                mTimeSpentIn.Gradient.Function = Inferno.Utilities.MarkerList.RFunctions.sqrt;


            }
            if (mCalculationInfo.TimeSpentOutside)
            {
                mTimeSpentOut = new RNumericImagePallete(mOutputGridArea.Grid, mBuddhabrotOut.GetGradient());
                mTimeSpentOut.SetToInitValue(0);

                mFormula.SetBuddhaGridArea(mOutputGridArea);
                mFormula.SetmTimeSpentOut(mTimeSpentOut);
                mTimeSpentOut.Gradient.Function = Inferno.Utilities.MarkerList.RFunctions.sqrt;
            }

        }
		public string mFileName;

		public string FileName
		{
			get
			{
				return mFileName;
			}
			set
			{
				mFileName = value;
			}
		}

		public void SaveToFile(string nFileName)
		{
			StreamWriter sw = new StreamWriter(nFileName);

			mInputGridArea.SaveToFile(sw, "Input");
			mOutputGridArea.SaveToFile(sw, "Output");
			sw.Close();
	
		}

		public void SaveToFile()
		{
			SaveToFile(mFileName);
		}

		public void LoadFromFile(string nFileName)
		{
			
			StreamReader sr = new StreamReader(nFileName);

			mInputGridArea.LoadFromFile(sr);
			mOutputGridArea.LoadFromFile(sr);
			
			sr.Close();

		}

		public void LoadFromFile()
		{
			LoadFromFile(mFileName);
		}

		public RGridArea InputGrid
		{
			get
			{
				return mInputGridArea;
			}
			set
			{
				mInputGridArea = value;

			}
		}

		internal RGridArea OutputGrid
		{
			get
			{
				return mOutputGridArea;
			}
			set
			{
				mOutputGridArea = value;;
			}
		}

		public RFractalFormulaAbstract GetFormula()
		{
			return mFormula;
		}

		public RNumericImageAbstract Mandelbrot
		{
			get
			{
				return mMandelBrot;
			}
			set
			{
                mMandelBrot = value;
			}
		}

        public RNumericImageAbstract BuddhabrotOut
        {
            get
            {
                return mBuddhabrotOut;
            }
            set
            {
                mBuddhabrotOut = value;
            }
        }

        public RNumericImageAbstract BuddhabrotIn
        {
            get
            {
                return mBuddhabrotIn;
            }
            set
            {
                mBuddhabrotIn = value;
            }
        }
        public RNumericImageAbstract CalculationPathOut
        {
            get
            {
                return mCalculationPathOut;
            }
            set
            {
                mCalculationPathOut = value;
            }
        }

        public RNumericImageAbstract CalculationPathIn
        {
            get
            {
                return mCalculationPathIn;
            }
            set
            {
                mCalculationPathIn = value;
            }
        }

        public RNumericImageAbstract TimeSpentOut
        {
            get
            {
                return mTimeSpentOut;
            }
            set
            {
                mTimeSpentOut = value;
            }
        }

        public RNumericImageAbstract TimeSpentIn
        {
            get
            {
                return mTimeSpentIn;
            }
            set
            {
                mTimeSpentIn = value;
            }
        }
        
        public void Calculate()
        {
            InitBuddbrotNumericImages();
            mFormula.Calculate();
        }
        public void RenderToDisk()
        {
            RNumericImageAbstract tMandelBrot = null;

            RNumericImageAbstract tBuddhabrotIn = null;
            RNumericImageAbstract tBuddhabrotOut = null;

            if (mCalculationInfo.MSet)
            {
                tMandelBrot = mMandelBrot;
            }
            if (mCalculationInfo.BuddhabrotInside)
            {
                tBuddhabrotIn = mBuddhabrotIn;
            }
            if (mCalculationInfo.BuddhabrotOutside)
            {
                tBuddhabrotOut = mBuddhabrotOut;
            }
            InitBuddbrotNumericImages();

            mFormula.Calculate();

            if (mCalculationInfo.MSet)
            {
                mMandelBrot.SaveBitMap();
                mMandelBrot = tMandelBrot;
            }
            if (mCalculationInfo.BuddhabrotInside)
            {
                mBuddhabrotIn.SaveBitMap();
                mBuddhabrotIn = tBuddhabrotIn;
            }
            if (mCalculationInfo.BuddhabrotOutside)
            {
                mBuddhabrotOut.SaveBitMap();
                mBuddhabrotOut = tBuddhabrotOut;
            }


        }
        public void CalculateToDisk()
        {
            RNumericImageAbstract tMandelBrot = null;

            RNumericImageAbstract tBuddhabrotIn = null;
            RNumericImageAbstract tBuddhabrotOut = null;

            if (mCalculationInfo.MSet)
            {
                tMandelBrot = mMandelBrot;
            }
            if (mCalculationInfo.BuddhabrotInside)
            {
                tBuddhabrotIn = mBuddhabrotIn;
            }
            if (mCalculationInfo.BuddhabrotOutside)
            {
                tBuddhabrotOut = mBuddhabrotOut;
            }
            InitBuddbrotNumericImages();

            mFormula.Calculate();

            if (mCalculationInfo.MSet)
            {
                mMandelBrot.SaveDataToFile();
                mMandelBrot = tMandelBrot;
            }
            if (mCalculationInfo.BuddhabrotInside)
            {
                mBuddhabrotIn.SaveDataToFile();
                mBuddhabrotIn = tBuddhabrotIn;
            }
            if (mCalculationInfo.BuddhabrotOutside)
            {
                mBuddhabrotOut.SaveDataToFile();
                mBuddhabrotOut = tBuddhabrotOut;
            }


        }
        
		public RAreaOptimizationArray Optimization
		{
			get
			{
				return mOptimization;
			}
		}
		
		}

//	}
}
