using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Windows.Forms;
using RakabrotPluginInterface.RCommon;
using CRFractal;
using RakabrotPluginInterface.RNumericImage;	


namespace Rakabrot
{
    public partial class RPictureBoxAreaPanel : System.Windows.Forms.Panel
    {
        public RPictureBoxAreaPanel()
        {
            InitializeComponent();

            mArea		   = new RArea(-2,-1,2,1);
			mAreaSelected  = new RArea(-2,-1,2,1);
            mImage         = new RNumericImagePallete();
			mState		   = RControlState.None;
            newState       = false;
			mMode		   = RPictureBoxAreaPanel.RControlMode.InputArea;
            
        }

        public RPictureBoxAreaPanel(IContainer container)
        {
            mArea = new RArea(-2, -1, 2, 1);
            mAreaSelected = new RArea(-2, -1, 2, 1);
            mImage = new RNumericImagePallete();
            mState = RControlState.None;
            newState = false;
            mMode = RPictureBoxAreaPanel.RControlMode.InputArea;
            
            container.Add(this);


            InitializeComponent();
        }
        	public enum RControlState
		{
			None = 0,
			FirstPoint = 1,
			SecondPoint = 2
		}
		public enum RControlMode
		{
			Optimization = 0,
			InputArea    = 1
		}

		RArea								mArea;
		RArea								mAreaSelected;
		Rakabrot.RControlGridArea			mProperty;
		Rakabrot.RControlAreaArray			mPropertyArray;
		RControlState						mState;
		RControlMode						mMode;
		public RNumericImageAbstract		mImage;
		bool								mShowOptimization;
        bool                                mHoldGridProportions;
        bool                                newState;
		protected RAreaOptimizationArray    mOptimizations;
		
		
//        System.ComponentModel.Container components = null;
	
		public  void SetProperty(Rakabrot.RControlGridArea	nProperty)
		{
			mProperty	= nProperty;
		}
		public  void SetArrayProperty(Rakabrot.RControlAreaArray	nPropertyArray)
		{
			mPropertyArray	= nPropertyArray;
		}
		
		private void NextState()
		{
			mState = (RControlState)(((int)mState +1) %3);
		}
		/// <summary> 
		/// Clean up any resources being used.
		/// </summary>
		

		#region Component Designer generated code
		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		
		#endregion


		
		void         PropertyRefresh()
		{
			switch(mMode)
			{
				case RControlMode.Optimization:
					if(mPropertyArray!= null)
						mPropertyArray.TotalRefresh();
					break;
				case RControlMode.InputArea:
					if(mProperty != null)
						mProperty.TotalRefresh();
					break;
			}
		}
		RVector      PictureToArea(RVector nVector)
		{
			double WidthStep  =  mArea.Width / this.Width; 
			double HeightStep = mArea.Height / this.Height ; 

			RVector Result = new RVector(nVector.X*WidthStep+ mArea.Left,
				nVector.Y *HeightStep+ mArea.Top
				);

			return Result;

		}
		RVector      AreaToPicture(RVector nVector)
		{
			double WidthStep  =  this.Width / mArea.Width; 
			double HeightStep =  this.Height / mArea.Height; 

			RVector Result = new RVector((nVector.X- mArea.Left)*WidthStep,
				(nVector.Y- mArea.Top) *HeightStep
				);
            
			return Result;
		}

        
		public void   DrawArea(RArea nArea, System.Drawing.Color mColor,bool Clear)
		{

            Graphics g;

                        g = this.CreateGraphics();


                        //            mImage.Width = mImage.Width;


            //g = Graphics.FromImage(this.Image);
            Pen myPen = new System.Drawing.Pen(mColor);

			if(Clear && mImage != null)
				mImage.DrawImage(g);
            if (nArea != null)
                g.DrawRectangle(myPen, (int)nArea.Left, (int)nArea.Top, (int)nArea.Width, (int)nArea.Height);
            g.Dispose();
        }
        public void DrawSelected(bool force)
		{

			RArea	Temp = GetPictureCoord();
            if (mState != RControlState.None || force)
            {
                DrawArea(Temp, System.Drawing.Color.Aqua, true);

                if (Temp != null)
                {
                    Graphics g;

                    //                    g = Graphics.FromImage(this.Image);
                                        g = this.CreateGraphics();
                    Brush myBrush = new System.Drawing.SolidBrush(System.Drawing.Color.Aqua);

                    g.FillRectangle(myBrush, (int)Temp.Left - 4, (int)Temp.Top - 4, 8, 8);
                    g.FillRectangle(myBrush, (int)Temp.Right - 4, (int)Temp.Top - 4, 8, 8);

                    g.FillRectangle(myBrush, (int)Temp.Left - 4, (int)Temp.Bottom - 4, 8, 8);
                    g.FillRectangle(myBrush, (int)Temp.Right - 4, (int)Temp.Bottom - 4, 8, 8);

                }


            }
             if (mShowOptimization)
                 DrawOptimizations();
            
		}
		void	DrawOptimizations()
		{
			int     MaxIndex = mOptimizations.GetMaxIndex();
			RArea	Temp;
			for(int i = 0; i< MaxIndex; i++)
			{
				Temp = GetPictureCoord(mOptimizations[i]);
				if(i == mOptimizations.Active)
				{
                    if (!((mMode == RPictureBoxAreaPanel.RControlMode.Optimization)
  					  ))
						DrawArea(Temp, System.Drawing.Color.Gold,false);
				}
				else 
						DrawArea(Temp, System.Drawing.Color.Gold,false);
					
  				    
					
			}
			
		}
		public  void ConvertAreaToPicture()
		{
			
		}
		
		public RArea  GetPictureCoord(RArea nArea)
		{
			if(nArea == null )
				return null;
			RArea Result = new RArea(AreaToPicture(nArea.GetTopLeft()),
				AreaToPicture(nArea.GetBottomRight()));
		    return Result;	
		}
		private RArea GetPictureCoord()
		{
			
			return GetPictureCoord(mAreaSelected);
		}

		private void RPictureBoxArea_MouseMove(object sender, System.Windows.Forms.MouseEventArgs e)
		{

            if (mState == RControlState.SecondPoint)
            {
                if (mMode == RPictureBoxAreaPanel.RControlMode.Optimization)
                {
                    mAreaSelected = mOptimizations.GetActive();
                }
                if (mAreaSelected != null)
                    mAreaSelected.SetBottomRight(PictureToArea(new RVector(e.X, e.Y)));
                DrawSelected(false);
            }
            
				
		}

        private void RPictureBoxArea_MouseDown(object sender, System.Windows.Forms.MouseEventArgs e)
        {

            if (!newState) 
            {
                NextState();
                if ((mState == RControlState.FirstPoint))
                {
                    if (mMode == RPictureBoxAreaPanel.RControlMode.Optimization)
                    {
                        mAreaSelected = mOptimizations.GetActive();
                    }
                    if (mAreaSelected != null)
                        mAreaSelected.SetTopLeft(PictureToArea(new RVector(e.X, e.Y)));
                    NextState();
                }

                
           
            }

            if ((mState == RControlState.SecondPoint))
            {
                if (mMode == RPictureBoxAreaPanel.RControlMode.Optimization)
                {
                    mAreaSelected = mOptimizations.GetActive();
                }
                if (mAreaSelected != null)
                    mAreaSelected.SetBottomRight(PictureToArea(new RVector(e.X, e.Y)));
            }
           
               
                DrawSelected(false);
                newState = true;

        }
        public void SetSelectedArea(RArea nArea)
		{
			if(nArea == null)
			{
				MessageBox.Show ("The Area is null", "Error", 
					MessageBoxButtons.OK);

			}
						
			mAreaSelected = nArea;
        }
		public	void SetNumericImage(RNumericImageAbstract nImage)
		{
			mImage = nImage;
		}
		public void ChangeAreaToSelectedArea()
		{
            if(mMode == RControlMode.InputArea)
			        mArea.Copy(mAreaSelected);
		}

		public RControlMode Mode
		{
			get
			{
				return mMode;
			}
			set
			{
                mState = RPictureBoxAreaPanel.RControlState.None;
				mMode = value;
			}
		}
		public void  SetOptimizationArray(RAreaOptimizationArray nOptimizations)
		{
			 mOptimizations = nOptimizations;
		}

		public bool ShowOptimization
		{
			get
			{
				return mShowOptimization;
			}
			set
			{
				mShowOptimization = value;
			}
		}
        public bool HoldGridProportions
        {
            get
            {
                return mHoldGridProportions;
            }
            set
            {
                mHoldGridProportions = value;
            }
        }

		
		

        

        private void RPictureBoxArea_MouseUp(object sender, MouseEventArgs e)
        {
            
                newState = false;
                mState = RControlState.None;
            }

        private void RPictureBoxArea_MouseLeave(object sender, EventArgs e)
        {
            mState = RControlState.None;
            newState = false;
        }

        private void RPictureBoxArea_Paint(object sender, PaintEventArgs e)
        {
            
            DrawSelected(true);
        
        }

        }


    }


